<?php
/**
 * Exit if accessed directly
 */

/// alm 20210418 21:48

if ( !defined( 'ABSPATH' ) ) {
    exit;
}

if ( class_exists( 'WPCOOP_Admin' ) &&
    !class_exists( 'WPCOOP_Admin_Settings' )) {

    class WPCOOP_Admin_Settings extends WPCOOP_Admin {

        public function __construct() {
            global $wpcoop;
            // copy some parameters from wpcoop
            //$this->plugin_version = $wpcoop->plugin_version; // not yet available
            $this->plugin_prefix = $wpcoop->plugin_prefix;
            $this->plugin_textdomain = $wpcoop->plugin_textdomain;
            $this->plugin_version = $wpcoop->plugin_version;

            $this->plugin_settings_capability=$wpcoop->plugin_settings_capability;

            $this->plugin_read_capability=$wpcoop->plugin_read_capability;
            $this->plugin_edit_capability=$wpcoop->plugin_edit_capability;
            $this->plugin_parameters_capability=$wpcoop->plugin_parameters_capability;

            $this->plugin_options_page = 'wpcoop_options';

            $this->plugin_metakey_options_coop_roles=$this->plugin_prefix.'options_coop_roles';

            // when admin initialisation, add settings
            add_action( 'admin_init', array( $this, 'init_options_page' ));

            // when in admin menu, add admin pages subpage and settings
            add_action( 'admin_menu', array( $this, 'setup_options_page' ));

        }
       /*
        private function get_plugin_version() {
          global $wpcoop;
          return $wpcoop->plugin_version;
        }
        */

        public function setup_options_page() {
            // add options page (parameters setup)
            $op=add_options_page( __( 'WpCoop setup', $this->plugin_textdomain ),
                              __( 'WpCoop setup', $this->plugin_textdomain ),
                              //only multisite superuser has all capabilities unless specifically denied
                              $this->plugin_settings_capability,
                              $this->plugin_options_page,
                              array($this, 'display_options_page')
                            );
            // add help page
            add_action('load-'.$op, array($this, 'op_help'));
        }

        public function op_help(){
          $overview_help = '<p>' . __('Options de WPCOOP',$this->plugin_textdomain) . '</p>' .
                          '<p>' . __('Ces options sont accessibles seulement aux personnes administrant le site, elles permettent essentiellement de définir les utilisateurs ayant l\'accès aux fonctionnalités du plugin (Roles), ainsi ques les options du plugin nécessitant une action de l\'administrateur du site. ',$this->plugin_textdomain) . '</p>'.
                          '<p>' . __('Les paramètres de fonctionnement de la société gérée par le plugin sont accessibles dans le menu "WpCoop paramètres", accessible dans le menu d\'administration principal, seulement aux opérateurs autorisés à paramétrer la société',$this->plugin_textdomain) . '</p>';

          $events_help = '<p>' . __("Events",$this->plugin_textdomain) . '</p>' .
                         '<p>' . __("Cet onglet permet d'activer ou pas l'affichage du menu 'WpCoop Events', accessible dans le menu d\'administration principal, seulement aux opérateurs autorisés et permettant de gérer les évènements dans WpCoop (Conseils, Réunions). Pas encore au point!!!, donc à laisser désactivé " ,$this->plugin_textdomain) ;
          $roles_help = '<p>' . __("Roles",$this->plugin_textdomain) . '</p>' .
                        '<p>' . __("Cet onglet permet de définir quels utilisateurs seront opérateurs du plugin, selon trois rôles: lecture, écriture, paramétrage." ,$this->plugin_textdomain).
                        '<p>' . __("A noter qu'un administrateur de site Wordpress unique devra être référencé pour accéder aux fonctionnalités, alors qu'un super-administrateur d'un Wordpress multisite aura tous les accès" ,$this->plugin_textdomain) ;

          $maps_help   = '<p>' . __("Maps",$this->plugin_textdomain) . '</p>' .
                         '<p>' . __('Permet d\'activer ou pas l\'utilisation de cartes des sociétaires',$this->plugin_textdomain) . '</p>'.
                         '<p>' . __('Si activé, il faut entrer les URL de l\'icon utilisée pour localiser les sociétaires, et l\'URL de la carte anonymisée et/ou l\'URL de la carte nominative',$this->plugin_textdomain) . '</p>';

          $promos_help = '<p>' . __("Promos",$this->plugin_textdomain) . '</p>' .
                         '<p>' . __("Permet l'affichage sur la page mon-compte du sociétaire les réductions ou avantages obtenus par le biais de la coopérative/société (peut nécessiter un développement informatique, donc à laisser inactif si ce développement n'est pas fait)" ,$this->plugin_textdomain).'</p>';

          $inline_help = '<p>' . __("In Line Operations",$this->plugin_textdomain) . '</p>' .
          '<p>' . __("Activation des opérations en ligne" ,$this->plugin_textdomain) . '</p>'.
          '<p>' . __("Activer ou pas les souscriptions en ligne ( nécessite un développement informatique, donc à laisser inactif si ce développement n'est pas fait)" ,$this->plugin_textdomain) . '</p>'.
          '<p>' . __("Activer ou pas les paiements en ligne ( nécessite un développement informatique, donc à laisser inactif si ce développement n'est pas fait)" ,$this->plugin_textdomain) . '</p>';

          get_current_screen()->add_help_tab(
          array(
           'id'      => 'overview',
           'title'   => __('Overview'),
           'content' => $overview_help,
              ));
          get_current_screen()->add_help_tab(
          array(
           'id'      => 'roles',
           'title'   => __('Roles'),
           'content' => $roles_help,
              ));
          get_current_screen()->add_help_tab(
          array(
           'id'      => 'events',
           'title'   => __('Events'),
           'content' => $events_help,
              ));
          get_current_screen()->add_help_tab(
          array(
           'id'      => 'maps',
           'title'   => __('Maps'),
           'content' => $maps_help,
              ));
          get_current_screen()->add_help_tab(
          array(
           'id'      => 'promos',
           'title'   => __('Promos'),
           'content' => $promos_help,
              ));
          get_current_screen()->add_help_tab(
          array(
           'id'      => 'inline',
           'title'   => __('In Line Operations'),
           'content' => $inline_help,
              ));
/*
          get_current_screen()->set_help_sidebar(
              '<p><strong>' . __('For more information:') . '</strong></p>' .
              '<p>' . __('<a href="https://codex.wordpress.org/Users_Your_Profile_Screen" target="_blank">Documentation on User Profiles</a>') . '</p>' .
              '<p>' . __('<a href="https://wordpress.org/support/" target="_blank">Support Forums</a>') . '</p>'
          );
*/
      }

        // options settings
        public function get_options_tabs($info='') {
          $medias=array(
                         'tabs'=> array(
                  array( 'slug'=>'roles', 'caption'=>__('Roles',$this->plugin_textdomain)),
                  array( 'slug'=>'events', 'caption'=>__('Events',$this->plugin_textdomain)),
                  array( 'slug'=>'maps', 'caption'=>__('Maps',$this->plugin_textdomain)),
                  array( 'slug'=>'promos', 'caption'=>__('Promos',$this->plugin_textdomain)),
                  array( 'slug'=>'inline', 'caption'=>__('In Line operations',$this->plugin_textdomain))
                                       ),
                        'options'=> array(
                  array( 'group'=>$this->plugin_metakey_options_coop_roles.'_group', 'name'=>$this->plugin_metakey_options_coop_roles,'callback'=>array($this, 'validate_options_coop_roles')),
                  array( 'group'=>$this->plugin_prefix.'options_events_group', 'name'=>$this->plugin_prefix.'options_events', 'callback'=>array($this, 'validate_options_events')),
                  array( 'group'=>$this->plugin_prefix.'options_maps_group', 'name'=>$this->plugin_prefix.'options_maps', 'callback'=>array($this, 'validate_options_maps')),
                  array( 'group'=>$this->plugin_prefix.'options_promos_group', 'name'=>$this->plugin_prefix.'options_promos', 'callback'=>array($this, 'validate_options_promos')),
                  array( 'group'=>$this->plugin_prefix.'options_inline_group', 'name'=>$this->plugin_prefix.'options_inline', 'callback'=>array($this, 'validate_options_inline'))
                                         ),
                         'actions'=> array(
//                  array( 'slug'=>'coop'),
//                  array( 'slug'=>'management')
                                          )
                        );
          $medias=apply_filters('wpcoop_options_tabs',$medias);
          switch($info) {
           case 'tabs':
            $ret=$medias['tabs'];
            break;
           case 'options':
            $ret=$medias['options'];
            break;
           case 'actions':
            $ret=$medias['actions'];
            break;
           default:
            $ret=$medias;
          }
          return $ret;
        }

        public function get_options_coop_roles(){
           return get_option($this->plugin_metakey_options_coop_roles);
        }

        public function get_capability_from_operator_type($type){
            switch ($type){
                case 'setter':
                    $cap=$this->plugin_parameters_capability;
                    break;
                case 'editor':
                    $cap=$this->plugin_edit_capability;
                    break;
                case 'read':
                default:
                    $cap=$this->plugin_read_capability;
            }
            return $cap;
        }


        public function get_plugin_readers(){
          return $this->get_plugin_operators('reader');
        }

        public function get_plugin_editors(){
          return $this->get_plugin_operators('editor');
        }

        public function get_plugin_setters(){
          return $this->get_plugin_operators('setter');
        }

        public function get_plugin_operators($type) {
            $ops = get_users(
                   array('role'=>$this->get_capability_from_operator_type($type),
                         'fields'=>'ID')
                        );
            return apply_filters($this->plugin_prefix.'get_plugin_operators', $ops, $type);
        }

        public function return_plugin_operators_lines($ids, $type) {
            // $ids is array of valid user ids, not empty
            $lines='';
            foreach ($ids as $id){
             $ud=get_userdata($id);
             $lines.='<tr><td>'.$id.'</td><td>'.esc_html($ud->user_login).'</td><td>'.esc_html($ud->display_name).'</td><td>'.esc_html($ud->user_nicename).'</td>'."\n";
             $lines.='<td><input type="submit" name="'.$this->plugin_metakey_options_coop_roles.'[revoke_'.$type.']['.$id.']" value="'.__("Revoke",$this->plugin_textdomain).'"</td></tr>'."\n";
            }
            return $lines;
        }

        public function return_plugin_operators_headers(){
            return  '<tr>
                      <th>'.__("ID",$this->plugin_textdomain).'</th>
                      <th>'.__("Login",$this->plugin_textdomain).'</th>
                      <th>'.__("Display Name",$this->plugin_textdomain).'</th>
                      <th>'.__("Nice Name",$this->plugin_textdomain).'</th>
                     </tr>';
        }

        public function get_plugin_add_operator($type){
            $operators=$this->get_plugin_operators($type);
            return '<tr>
            <td colspan="3"  style="border-width: 1px; border-style: solid none none none" >'.__("Ajouter un opérateur",$this->plugin_textdomain).'</td>
            <td style="border-width: 1px; border-style: solid none none none">'.
            //https://developer.wordpress.org/reference/functions/wp_dropdown_users/
            wp_dropdown_users(array('echo'=>0, 'multi'=>1, 'exclude'=>$operators, 'name'=>$this->plugin_metakey_options_coop_roles.'['.$type.']')).'
            </td>
            <td style="border-width: 1px; border-style: solid none none none"><input type="submit" name="'.$this->plugin_metakey_options_coop_roles.'[add_'.$type.']" value="'.__("Add",$this->plugin_textdomain).'"></td></tr>';
        }

        // Renders the options page. Uses the settings API (options validation, checking and storing by WP).
        public function display_options_page() {
          /////// get active tab
          if (!current_user_can($this->plugin_settings_capability)) {
            _e('Les réglages de WPCOOP vous sont inaccessibles, consultez votre webmaster',$this->plugin_textdomain);
          } else {
          $tabs= $this->get_options_tabs('tabs');
          $active_tab='roles';
          if( isset( $_GET[ 'tab' ] ) ) {
            foreach($tabs as $tab) {
              if($_GET[ 'tab' ]==$tab['slug']) $active_tab=$tab['slug'];
            }
          }

    /////// displays tabs
    ?>
    <div class="wrap">
        <div class="icon32" id="icon-options-general"><br /></div>
        <h2>
            <?php _e('WpCoop V: '.$this->plugin_version, $this->plugin_textdomain) ?>
        </h2>
        <h2 class="nav-tab-wrapper">
        <?php
        if (!empty($tabs)) {
          foreach ($tabs as $tab) {
            $addclass= ($tab['slug']==$active_tab)? 'nav-tab-active' : '';
            ?>
            <a href="?page=<?php echo $this->plugin_options_page; ?>&tab=<?php echo $tab['slug']; ?>" class="nav-tab <?php echo $addclass; ?>"><?php echo $tab['caption']; ?> </a>
            <?php
          }
        }
        ?>
        </h2>
    </div>
    <?php
    switch ($active_tab) {
         case 'roles':
            $options = get_option($this->plugin_metakey_options_coop_roles);
            ?>
            <form method="post" action="options.php" >
              <?php settings_fields($this->plugin_metakey_options_coop_roles.'_group'); ?>
              <h3><?php _e('Roles', $this->plugin_textdomain) ?></h3>
              <div>
                 <table width="100%" style="border: solid 1px">
                  <tr><th colspan="5"><?php _e("Opérateurs pouvant lire les données",$this->plugin_textdomain) ?></th></tr>
                <?php
                 if (!empty($this->get_plugin_readers())) {
                   echo $this->return_plugin_operators_headers();
                   echo $this->return_plugin_operators_lines($this->get_plugin_readers(), 'reader');
                 }
                 echo $this->get_plugin_add_operator('reader');
                ?>
                 </table><br/>
                 <table width="100%" style="border: solid 1px">
                  <tr><th colspan="5"><?php _e("Opérateurs pouvant modifier les données",$this->plugin_textdomain) ?></th></tr>
                    <?php
                     if (!empty($this->get_plugin_editors())) {
                       echo $this->return_plugin_operators_headers();
                       echo $this->return_plugin_operators_lines($this->get_plugin_editors(), 'editor');
                     }
                     echo $this->get_plugin_add_operator('editor');
                    ?>
                </table><br/>
                <table width="100%" style="border: solid 1px">
                  <tr><th colspan="5"><?php _e("Opérateurs pouvant paramétrer la société",$this->plugin_textdomain) ?></th></tr>
                    <?php
                     if (!empty($this->get_plugin_setters())) {
                      echo $this->return_plugin_operators_headers();
                      echo $this->return_plugin_operators_lines($this->get_plugin_setters(), 'setter');
                     }
                      echo $this->get_plugin_add_operator('setter');
                    ?>
                </table><br/>
              </div>
             </form>
             <?php
             break;
         case 'events':
            /// display form for events
            $options= get_option($this->plugin_prefix.'options_events', array());
            ?>
            <div>
             <form method="post" action="options.php">
                 <?php settings_fields($this->plugin_prefix.'options_events_group'); ?>
                 <h3><?php _e('Events', $this->plugin_textdomain) ?></h3>
                 <div>
                  <table class="form-table">
                    <tr>
                     <td><?php _e("Events menu enabled",$this->plugin_textdomain) ?></td>
                     <?php
                     $checked=(isset($options['menu_enabled']) && $options['menu_enabled']=='y')? 'checked="checked"' : '';
                     ?>
                     <td><input type="checkbox" name="<?php echo $this->plugin_prefix.'options_events[menu_enabled]'; ?>" <?php echo $checked; ?>/></td>
                     <td><?php _e("If enabled, menu visible in main admin menu",$this->plugin_textdomain) ?></td>
                    </tr>
                  </table>
                  <p class="submit">
                     <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
                  </p>
                 </div>
            </form>
            </div>
            <?php
          break;
          case 'maps':
             /// display form for maps
             $options=get_option($this->plugin_prefix.'options_maps',array());
             ?>
             <div>
              <form method="post" action="options.php">
                  <?php settings_fields($this->plugin_prefix.'options_maps_group'); ?>
                  <h3><?php _e('Maps options', $this->plugin_textdomain) ?></h3>
                  <div>
                   <table class="form-table">
                     <tr>
                      <td><?php _e("Maps enabled",$this->plugin_textdomain) ?></td>
                      <?php
                      $checked=(isset($options['maps_enabled']) && $options['maps_enabled']=='y')? 'checked="checked"' : '';
                      ?>
                      <td><input type="checkbox" name="<?php echo $this->plugin_prefix.'options_maps[maps_enabled]'; ?>" <?php echo $checked; ?>/></td>
                      <td><?php _e("If enabled, maps URL to enter below",$this->plugin_textdomain) ?></td>
                       </tr>
                       <?php if (!empty($checked)) { ?>
                        <tr>
                         <td><?php _e("URL map icon",$this->plugin_textdomain) ?></td>
                         <td><input type="text" name="<?php echo $this->plugin_prefix.'options_maps[url_map_icon]'; ?>" value="<?php echo esc_attr($options['url_map_icon']); ?>" /></td>
                        </tr>
                        <tr>
                         <td><?php _e("URL anonymous map",$this->plugin_textdomain) ?></td>
                         <td><input type="text" name="<?php echo $this->plugin_prefix.'options_maps[url_anonymous_map]'; ?>" value="<?php echo esc_attr($options['url_anonymous_map']); ?>" /></td>
                        </tr>
                        <tr>
                         <td><?php _e("URL full map",$this->plugin_textdomain) ?></td>
                         <td><input type="text" name="<?php echo $this->plugin_prefix.'options_maps[url_full_map]'; ?>" value="<?php echo esc_attr($options['url_full_map']); ?>" /></td>
                        </tr>
                      <?php } ?>
                      </tr>
                   </table>
                   <p class="submit">
                      <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
                   </p>
                  </div>
             </form>
             </div>
             <?php
           break;
         case 'inline':
           /// display form for inline
           $options=get_option($this->plugin_prefix.'options_inline', array());
           ?>
           <div>
            <form method="post" action="options.php">
                <?php settings_fields($this->plugin_prefix.'options_inline_group'); ?>
                <h3><?php _e('In line operations', $this->plugin_textdomain) ?></h3>
                <div>
                 <table class="form-table">
                   <tr>
                    <td><?php _e("Souscription en ligne active",$this->plugin_textdomain) ?></td>
                    <?php
                    $checked=(isset($options['inline_subs']) && $options['inline_subs']=='y')? 'checked="checked"' : '';
                    ?>
                    <td><input type="checkbox" name="<?php echo $this->plugin_prefix.'options_inline[inline_subs]'; ?>" <?php echo $checked; ?>/></td>
                    <td><?php _e("Activer la souscription en ligne",$this->plugin_textdomain) ?></td>
                   </tr>
                   <tr>
                    <td><?php _e("Paiement en ligne actif",$this->plugin_textdomain) ?></td>
                    <?php
                    $checked=(isset($options['inline_pay']) && $options['inline_pay']=='y')? 'checked="checked"' : '';
                    ?>
                    <td><input type="checkbox" name="<?php echo $this->plugin_prefix.'options_inline[inline_pay]'; ?>" <?php echo $checked; ?>/></td>
                    <td><?php _e("Activer le paiement en ligne",$this->plugin_textdomain) ?></td>
                   </tr>
                 </table>
                 <p class="submit">
                    <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
                 </p>
                </div>
           </form>
           </div>
           <?php
           break;
          case 'promos':
            /// display form for promos
            $options= get_option($this->plugin_prefix.'options_promos', array());
            ?>
            <div>
             <form method="post" action="options.php">
                 <?php settings_fields($this->plugin_prefix.'options_promos_group'); ?>
                 <h3><?php _e('Promos', $this->plugin_textdomain) ?></h3>
                 <div>
                  <table class="form-table">
                    <tr>
                     <td><?php _e("Promos menu enabled",$this->plugin_textdomain) ?></td>
                     <?php
                     $checked=(isset($options['menu_enabled']) && $options['menu_enabled']=='y')? 'checked="checked"' : '';
                     ?>
                     <td><input type="checkbox" name="<?php echo $this->plugin_prefix.'options_promos[menu_enabled]'; ?>" <?php echo $checked; ?>/></td>
                     <td><?php _e("If enabled, menu visible in main admin menu",$this->plugin_textdomain) ?></td>
                    </tr>
                  </table>
                  <p class="submit">
                     <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
                  </p>
                 </div>
                </form>
                </div>
                <?php
           break;
          }//switch
         }//no rights to display settings page
        }//display options page

        function init_options_page() {
            // get options
            $options=$this->get_options_tabs('options');
            // registers options groups
            foreach($options as $option) {
             register_setting($option['group'],$option['name'],$option['callback']);
            }
        }

        public function validate_user($id){
            $id=(int) $id;
             if (get_user_by('id', $id)){
              return $id;
             } else {
              return 0;
             }
        }

        public function validate_options_coop_roles($input){
          $id=0;
          $type='';
          $ok=false;
          if (isset($input['add_reader']) && isset($input['reader'])){
             $type='reader';
             $id=$this->validate_user($input[$type]);
          } elseif (isset($input['add_editor']) && isset($input['editor'])){
             $type='editor';
             $id=$this->validate_user($input[$type]);
          } elseif (isset($input['add_setter']) && isset($input['setter'])){
             $type='setter';
             $id=$this->validate_user($input[$type]);
          } else {
           $ok=true;
           if (isset($input['revoke_reader'])) {
            $user = new WP_User(key($input['revoke_reader']) );
            $user->remove_cap($this->get_capability_from_operator_type('reader'));
           } elseif (isset($input['revoke_editor'])) {
            $user = new WP_User(key($input['revoke_editor']) );
            $user->remove_cap($this->get_capability_from_operator_type('editor'));
           } elseif (isset($input['revoke_setter'])) {
            $user = new WP_User(key($input['revoke_setter']) );
            //$user->remove_cap('wpcoop_set_capability');
            $user->remove_cap($this->get_capability_from_operator_type('setter'));
           }
          }
          if($id && !empty($type)) {
              $user = new WP_User($id );
              $user->add_cap($this->get_capability_from_operator_type($type));
              if ($type=='editor'){
               $user->add_cap($this->get_capability_from_operator_type('read'));
              }
          } elseif(!$ok) {
              add_settings_error( $this->plugin_metakey_options_coop_roles, 'invalid-user', __("Invalid User", $this->plugin_textdomain ));
          }
          return 'last roles edit '.get_current_user_id().' '.date('YmdHis');
        }


        public function validate_options_events($input) {
          if (isset($input['menu_enabled'])) $input['menu_enabled']='y'; else $input=array();
          return apply_filters($this->plugin_prefix.'validate_options_events', $input);
        }

        public function validate_options_maps($input) {
          //called twice if option doesn't email_exists
          // https://core.trac.wordpress.org/ticket/21989
          $options=get_option($this->plugin_prefix.'options_maps',array());
          $output=array();
          if (empty($input)){
            //user validates blank page, keeps olds values if any
            $output=$options;
          } elseif (!isset($input['maps_enabled'])){
            //disabling maps, keeps values in form
            $output=$options;
            if (isset($output['maps_enabled'])) unset($output['maps_enabled']);
          } elseif (!isset($options['maps_enabled']) && isset($input['maps_enabled'])){
            //enabling maps, was not, so no new values
            $output=$options;
            $output['maps_enabled']='y';
          } else {
            //enabling maps, check for url
            $output['maps_enabled']='y';
            foreach($input as $k=>$vi){
              if ($k=='maps_enabled') continue;
              $output[$k]= (string) $vi;
              if(!wp_http_validate_url( $output[$k] )) {
                 add_settings_error( $this->plugin_prefix.'options_maps', 'invalid-url', __('You have entered an invalid URL.', $this->plugin_textdomain ));
              }
            }
          }
          return apply_filters($this->plugin_prefix.'validate_options_maps', $output);
        }

        public function validate_options_promos($input) {
          if (isset($input['menu_enabled'])) $input['menu_enabled']='y';
          return apply_filters($this->plugin_prefix.'validate_options_promos', $input);
        }

        public function validate_options_inline($input) {
          if (isset($input['inline_subs'])) $input['inline_subs']='y'; //else unset($input['menu_enabled']);
          if (isset($input['inline_pay'])) $input['inline_pay']='y'; //else unset($input['menu_enabled']);
          return apply_filters($this->plugin_prefix.'validate_options_inline', $input);
        }

    } // end class
} // end if class exists
