<?php
/**
 * Exit if accessed directly
 */
if ( !defined( 'ABSPATH' ) ) {
    exit; 
}
if ( !class_exists( 'WordPress_Cooperative_Test' ) ) {

    class WordPress_Cooperative_Test {

        public function __construct() {
            global $wpcoop;
            // copy some parameters from wpcoop
            //$this->plugin_version = $wpcoop->plugin_version; // not yet available
            $this->plugin_prefix = $wpcoop->plugin_prefix;
            $this->plugin_textdomain = $wpcoop->plugin_textdomain; 
            $this->plugin_version = $wpcoop->plugin_version; 
            
            $this->plugin_read_capability=$wpcoop->plugin_read_capability;
            $this->plugin_edit_capability=$wpcoop->plugin_edit_capability;
            $this->plugin_set_capability=$wpcoop->plugin_set_capability;
            $this->plugin_settings_capability=$wpcoop->plugin_settings_capability;
            
            
            $this->class_test_slug=$this->plugin_prefix.'test';

            $this->plugin_options_page = 'wpcoop_options';
            
            $this->plugin_metakey_options_coop_roles=$this->plugin_prefix.'options_coop_roles';

            // when admin initialisation, add settings
            add_action( 'admin_init', array( $this, 'init_options_page' ));
            
            // when in admin menu, add admin pages subpage and settings
            add_action( 'admin_menu', array( $this, 'setup_options_page' ));
            
        }
        
        function init_options_page() {
            // get options
            $options=$this->get_options_tabs('options');
            // registers options groups
            foreach($options as $option) {
             register_setting($option['group'],$option['name'],$option['callback']);
            }
        }
        

     
        public function setup_options_page() {            
            // add options page (parameters setup)
             $mp=add_menu_page( __('WpCoop Test',$this->plugin_textdomain),
                           __('WpCoop Test',$this->plugin_textdomain),
                           $this->plugin_read_capability,
                           $this->class_test_slug,
                           array($this, 'test_page'),
                           'dashicons-bank', // icone  url 
                           '3' // position
                         );
             add_action('load-'.$mp, array($this, 'page_help'));

        }
        // options settings
            
    
        public function get_options_tabs($info='') {
          $medias=array(
                         'tabs'=> array(
                  array( 'slug'=>'tab1', 'caption'=>__('Tab1',$this->plugin_textdomain)),
                  array( 'slug'=>'tab2', 'caption'=>__('Tab2',$this->plugin_textdomain)),
                                       ),
                        'options'=> array(
                  array( 'group'=>$this->plugin_prefix.'options_tab1_group', 'name'=>$this->plugin_prefix.'options_tab1','callback'=>array($this, 'validate_options_tab1')),
                  array( 'group'=>$this->plugin_prefix.'options_tab2_group', 'name'=>$this->plugin_prefix.'options_tab2','callback'=>array($this, 'validate_options_tab2')),
                                         ),
                         'actions'=> array(
                  array( 'slug'=>'tab1'),
                  array( 'slug'=>'tab2')
                                          )
                        );
          $medias=apply_filters('wpcoop_test_tabs',$medias);
          switch($info) {
           case 'tabs':
            $ret=$medias['tabs'];
            break;
           case 'options':
            $ret=$medias['options'];
            break;
           case 'actions':
            $ret=$medias['actions'];
            break;
           default:
            $ret=$medias;
          }
          return $ret;
        } 
        
        public function test_page(){
          if (!current_user_can($this->plugin_settings_capability)) {
            _e('Les réglages de WPCOOP vous sont inaccessibles, consultez votre webmaster',$this->plugin_textdomain);
          } else {
          $tabs= $this->get_options_tabs('tabs');
          $active_tab='tab1';
          if( isset( $_GET[ 'tab' ] ) ) {
            foreach($tabs as $tab) {
              if($_GET[ 'tab' ]==$tab['slug']) $active_tab=$tab['slug'];
            }
          }
            /////// displays tabs
            ?>
            <div class="wrap">
                <div class="icon32" id="icon-options-general"><br /></div>
                <h2>
                    <?php _e('WpCoop V: '.$this->plugin_version, $this->plugin_textdomain) ?>
                </h2>
                <h2 class="nav-tab-wrapper">
                <?php
                settings_errors();
                if (!empty($tabs)) {
                  foreach ($tabs as $tab) {
                    $addclass= ($tab['slug']==$active_tab)? 'nav-tab-active' : '';
                    ?>
                    <a href="?page=<?php echo $this->class_test_slug; ?>&tab=<?php echo $tab['slug']; ?>" class="nav-tab <?php echo $addclass; ?>"><?php echo $tab['caption']; ?> </a>
                    <?php
                  }
                }
                ?>
                </h2>
            </div>
            
            <?php
            switch ($active_tab) {
                case 'tab1':       
                $options = get_option($this->plugin_prefix.'options_tab1');
                ?>
                <form method="post" action="options.php" >
                  <?php settings_fields($this->plugin_prefix.'options_tab1_group'); ?>
                  <h3><?php _e('tab1 options', $this->plugin_textdomain) ?></h3>
                  <div>
                    <table class="form-table">
                     <tr>
                        <td><?php _e("tab1 data1",$this->plugin_textdomain) ?></td>
                         <td><select name="<?php echo $this->plugin_prefix.'options_tab1'; ?>" >
                                <option value="toto" <?php echo (isset($options) && $options=='toto')? 'selected="selected"' : '' ?> >TOTO</option>
                                <option value="tata" <?php echo (isset($options) && $options=='tata')? 'selected="selected"' : '' ?> >TATA</option>
                             </select>
                         </td>
                     </tr>
                      </table>
                    </div>
                    <p class="submit">
                        <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
                    </p>
                </form>  
                <?php
                  break;
                case 'tab2':
                $options = get_option($this->plugin_prefix.'options_tab2');
                ?>
                <form method="post" action="options.php" >
                  <?php settings_fields($this->plugin_prefix.'options_tab2_group'); ?>
                  <h3><?php _e('tab2 options', $this->plugin_textdomain) ?></h3>
                  <div>
                    <table class="form-table">
                     <tr>
                        <td><?php _e("Tab2 ",$this->plugin_textdomain) ?></td>
                         <td>
                          <input type="text" name="<?php echo $this->plugin_prefix.'options_tab2'; ?>" value="<?php echo (isset($options))? esc_attr($options) : ''; ?>" /> 
                             
                         </td>
                     </tr>
                      </table>
                    </div>
                    <p class="submit">
                        <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
                    </p>
                </form>  
                <?php
                break;        
             }
          }
        }
   
        public function page_help(){
            $overview_help = '<p>' . __('Contact site administrator',$this->plugin_textdomain) . '</p>
            <p>' . __('Texte help',$this->plugin_textdomain) . '</p>' ;
            get_current_screen()->add_help_tab( array(
	           'id'      => 'overview',
	           'title'   => __('Help'),
	           'content' => $overview_help,
            ) );                                        
        }                      

        public function validate_options_tab1($input){
            //add_settings_error( $this->plugin_prefix.'tab1', 'tab1_error', __('Tab1 error.', $this->plugin_textdomain ));

            return $input;
        }
        
        public function validate_options_tab2($input){
            //add_settings_error( $this->plugin_prefix.'tab2', 'tab2_error', __('Tab2 error.', $this->plugin_textdomain ));

            return $input;
        }
        
        
        
        
        
        public function simulate_settings_messages($index, $text){
          // when sending data directly (not to options.php) validate and error messages don't fire
          echo '<div id="'.$index.'" class="notice notice-error settings-error is-dismissible"
                    <p><strong>'.$text.'</strong></p>
                    <button class="notice-dismiss" type="button">
                    <span class="screen-reader-text">Ne pas tenir compte de ce message</span>
                    </button>
                </div>';                   
        }
        
        function set_error_messages_after_uploading_file(){
            
            
            
        }
        
        
        // TODO admin messages
        
        // TODO merge with admin function
        public function validate_date($maybedate) {
          return ( empty($maybedate) || strlen($maybedate)!=10 || 
                  !checkdate( (int) substr($maybedate, 3, 2), (int) substr($maybedate, 0, 2), (int) substr($maybedate, 6, 4) ) )? false : true;   
                        
        }

        public function validate_options_subs($input) {
          // not usable becaus $_FILES empty
          return apply_filters($this->plugin_prefix.'validate_options_subs', $input);  
        }

        public function validate_user($id){
            $id=(int) $id;
             if (get_user_by('id', $id)){
              return $id;   
             } else {
              return 0;   
             }            
        }
        
        public function get_capability_from_operator_type($type){
            switch ($type){
                case 'setter':
                    $cap=$this->plugin_set_capability;
                    break;
                case 'editor':
                    $cap=$this->plugin_edit_capability;
                    break;
                case 'read':
                default:    
                    $cap=$this->plugin_read_capability;
            }
            return $cap;
        }
        
        public function validate_options_coop_roles($input){
          $id=0;
          $type='';
          $ok=false;
          if (isset($input['add_reader']) && isset($input['reader'])){
             $type='reader';  
             $id=$this->validate_user($input[$type]);             
          } elseif (isset($input['add_editor']) && isset($input['editor'])){
             $type='editor';  
             $id=$this->validate_user($input[$type]);              
          } elseif (isset($input['add_setter']) && isset($input['setter'])){
             $type='setter'; 
             $id=$this->validate_user($input[$type]);              
          } else {
           $ok=true;   
           if (isset($input['revoke_reader'])) {
            $user = new WP_User(key($input['revoke_reader']) );
            $user->remove_cap($this->get_capability_from_operator_type('reader'));   
           } elseif (isset($input['revoke_editor'])) {
            $user = new WP_User(key($input['revoke_editor']) );
            $user->remove_cap($this->get_capability_from_operator_type('editor'));   
           } elseif (isset($input['revoke_setter'])) {
            $user = new WP_User(key($input['revoke_setter']) );
            $user->remove_cap($this->get_capability_from_operator_type('setter'));   
           }
          }
          if($id && !empty($type)) {
              $user = new WP_User($id );
              $user->add_cap($this->get_capability_from_operator_type($type));
              if ($type=='editor'){
               $user->add_cap($this->get_capability_from_operator_type('read'));
              }
          } elseif(!$ok) {
              add_settings_error( $this->plugin_metakey_options_coop_roles, 'invalid-user', __("Invalid User", $this->plugin_textdomain ));
          }
          return 'last roles edit '.get_current_user_id().' '.date('YmdHis');  
        }
        
        public function validate_options_coop($input) {
          //wlogt(LOG, '$input', $input);
          foreach($input as $k=>$v) {
           //wlog(LOG, $k);   
           if ($k=='logo_file') {
               
               wlogt(LOG, '$_FILES', $_FILES);
               
           } else {     
            if (!is_string($v)) $input[$k]= (string) $v;  
            if ($k=='type' && !in_array($v, array('scic','scop')))  $input[$k]='scop';      
            if ($k=='form' && !in_array($v, array('sa','sarl', 'sas')))  $input[$k]='sa'; 
            if ($k=='created') $input[$k]= ( $this->validate_date($v) )?  $v : ''; 
           }
          }  
            
          return apply_filters($this->plugin_prefix.'validate_options_coop', $input);  
        }
        
        public function validate_options_cats($input) {
          $options = get_option($this->plugin_prefix.'options_cats');                                
          //wlogt(LOG, '$input', $input);
          //$catindex = (!isset($input['catindex']))? 0 : (int) $input['catindex']; 
          //$catindex=array_key_first($input);
          $catindex=array_keys($input)[0];
          if (isset($input[$catindex]['delcat'])) {
              $options[$catindex]['deleted']='y';   
          } else {
              foreach($input[$catindex] as $k=>$v) {
                wlog(LOG, $k); 
                if($k=='addcat' || $k=='delcat' || $k=='modcat' ) continue; 
                $options[$catindex][$k]= (string) $v;  
              }
          }
          return apply_filters($this->plugin_prefix.'validate_options_cats', $options);  
        }
        
        public function validate_options_shares($input) {
          foreach($input as $k=>$v) {
            if (!is_string($v)) $input[$k]= (string) $v;  
          }  
          $input['price']= ($input['price']==0 || empty($input['price']) || intval($input['price'])!=$input['price'])? 100 : $input['price'];
          return apply_filters($this->plugin_prefix.'validate_options_shares', $input); 
        }
        
        public function validate_options_management($input) {
            
          foreach($input as $k=>$v) {
              if (!is_string($v)) $input[$k]= (string) $v;

              if ( $k== 'council_min' || $k== 'council_max') {
                    $input[$k]= (string) absint($v);
              }
              if ( $k== 'council_min' && isset($input['council_min']) && isset($input['council_max']) && !( absint($input['council_max'])+1 > absint($input['council_min']) ) ) {
                 add_settings_error( $this->plugin_prefix.'options_management', 'invalid-council', __("Maximum number must be greater or equal to minimum", $this->plugin_textdomain ));
                 $input['council_min']=absint($input['council_max']);
              }
              if ( $k== 'council_shareholders' && $input['council_shareholders']!='y' && $input['council_shareholders']!='n'){
                 add_settings_error( $this->plugin_prefix.'options_management', 'invalid-council', __("Choice must be Yes or No", $this->plugin_textdomain ));
                 $input[$k]='y'; 
              }
              
              if ( $k== 'auditor' && $input['auditor']!='y' && $input['auditor']!='n'){
                 add_settings_error( $this->plugin_prefix.'options_management', 'invalid-auditor', __("Auditor choice must be Yes or No", $this->plugin_textdomain ));
                 $input[$k]='y'; 
              }
              if ( $k== 'auditor' && $input['auditor']=='y' && (empty($input['auditor_name']) || empty($input['auditor_email']) )) {
                 add_settings_error( $this->plugin_prefix.'options_management', 'invalid-auditor', __("Auditor name or email can't be empty.", $this->plugin_textdomain ));
               }
              if ( $k== 'auditor_email' && !is_email($v) ) {
                if (!empty($input['auditor_name']) && !empty($v)) {  
                 add_settings_error( $this->plugin_prefix.'options_management', 'invalid-email', __('You have entered an invalid e-mail address.', $this->plugin_textdomain ));
                }
                $input[$k]='';  
              } 
          }  
          return apply_filters($this->plugin_prefix.'validate_options_management', $input); 
        }
        
        public function validate_options_meetings($input) {
          foreach($input as $k=>$v) {
            if (!is_string($v)) $input[$k]= (string) $v;  
          }  
            
          return apply_filters($this->plugin_prefix.'validate_options_meetings', $input); 
        }
        
        public function validate_options_councils($input) {
          $opts_man=get_option($this->plugin_prefix.'options_management');
          $options=get_option($this->plugin_prefix.'options_councils'); 
          // delete council member     
          if (isset($input['council_members_delete'])) {
                unset ($options['council_members'][intval(key($input['council_members_delete']))] );
          } elseif (isset($input['council_members_add'])) { 
          // add council member
            //@TODO verify uid is ok (existing user or partner) && uid is not already present
            $isin=false;  
            foreach($options['council_members'] as $m) {
                if ($m['uid']==intval($input['council_members']['uid'])) {
                    $isin=true;
                }
            } 
            if (count($options['council_members'])<$opts_man['council_max'] && !$isin) {
              $options['council_members'][]=array('role' => (string) $input['council_members']['role'], 'uid'=>intval($input['council_members']['uid']));
            } else {
              if(count($options['council_members'])==$opts_man['council_max']) {
                  add_settings_error( $this->plugin_prefix.'options_councils', 'council_max_exceeded', __('Maximum number of members exceeded.', $this->plugin_textdomain ));
              }
              if ($isin) {
                  add_settings_error( $this->plugin_prefix.'options_councils', 'user_isin', __('Already member of council.', $this->plugin_textdomain ));
              }
            }
          }
          return apply_filters($this->plugin_prefix.'validate_options_councils', $options); 
        }
        
        public function validate_options_maps($input) {
          foreach($input as $k=>$v) {
            $input[$k]= (string) $v;   
            if ( ! wp_http_validate_url( $input[$k] )) {
               add_settings_error( $this->plugin_prefix.'options_map', 'invalid-url', __('You have entered an invalid URL.', $this->plugin_textdomain ));
            } 
          }  
          return apply_filters($this->plugin_prefix.'validate_options_maps', $input); 
        }
        
        public function validate_options_promos($input) {
          wlogt(LOG, 'input', $input);    
          foreach($input as $k=>$v) {
            if (in_array($k, array('monkitsolaire'))) { $input[$k]= 'y'; } else { unset($input[$k]); } 
            //if ( ! wp_http_validate_url( $input[$k] )) {
            //   add_settings_error( $this->plugin_prefix.'options_map', 'invalid-url', __('You have entered an invalid URL.', $this->plugin_textdomain ));
            //}
          }
            //update_option($this->plugin_prefix.'options_promos', $promos);    
          return apply_filters($this->plugin_prefix.'validate_options_promos', $input); 
        }

    } // end class
} // end if class exists
