<?php
/*
Plugin Name: WpCoop
Plugin URI: http://pep2a.fr/wpcoop-plugin-de-gestion-de-cooperative/
Description: WpCoop plugin permet la gestion d'une cooperative (principalement SCIC SAS) ou de société s'en rapprochant : après activation du plugin, aller dans les réglages du site puis WpCoop réglages pour définir les options essentielles du fonctionnement, principalement les utilisateurs autorisés à gérer la coopérative. Les utilisateurs concernés verront apparaitre sur leur tableau de bord des menus correspondant à leurs rôles. Ce plugin est fourni sans aucune garantie et est utilisable dans le cadre d'une license Creative Common Share Alike <a href="https://creativecommons.org/licenses/by-sa/4.0/deed.fr">https://creativecommons.org/licenses/by-sa/4.0/deed.fr</a>. Ce plugin est entièrement gratuit, cependant des contributions <a href="https://pep2a.fr/dons/">en dons</a> ou <a href="https://pep2a.fr/souscrire/">en souscription</a> à notre coopérative  ainsi que toute aide seront les bienvenus.
Version: 0.23
Author: alain messin
*/

// alm 20210418 22:20

/*  Copyright 2017 pep2a
    licence Creative Common Share Alike, ce qui signifie  principalement que vous pouvez les utiliser comme vous voulez, mais que les logiciels les utilisant doivent être partagés avec la même licence.

    https://creativecommons.org/licenses/by-sa/4.0/
    https://creativecommons.org/licenses/by-sa/4.0/deed.fr

    Attribution — Vous devez créditer l'Œuvre, intégrer un lien vers la licence et indiquer si des modifications ont été effectuées à l'Oeuvre. Vous devez indiquer ces informations par tous les moyens raisonnables, sans toutefois suggérer que l'Offrant vous soutient ou soutient la façon dont vous avez utilisé son Oeuvre.

    Partage dans les Mêmes Conditions — Dans le cas où vous effectuez un remix, que vous transformez, ou créez à partir du matériel composant l'Oeuvre originale, vous devez diffuser l'Oeuvre modifiée dans les même conditions, c'est à dire avec la même licence avec laquelle l'Oeuvre originale a été diffusée.

    Pas de restrictions complémentaires — Vous n'êtes pas autorisé à appliquer des conditions légales ou des mesures techniques qui restreindraient légalement autrui à utiliser l'Oeuvre dans les conditions décrites par la licence.

*/

/**
 * Exit if accessed directly
 */
if ( !defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Base class
 */
if ( !class_exists( 'WordPress_Cooperative' ) ) {

    final class WordPress_Cooperative {

        //public static $plugin_prefix;
        //public static $plugin_url;
        //public static $plugin_dir_path;
        //public static $plugin_basefile;
        //public static $plugin_basefile_path;
        //public static $plugin_textdomain;

        public $docs_params;

        public $plugin_version;
        public $writepanel;
        public $settings;
        public $print;
        public $docs;
        public $theme;

        /**
         * Constructor
         */
        public function __construct() {
            // defines some variables

            $this->plugin_prefix = 'wpcoop_';
            $this->usersdata_prefix='userdata_';
            $this->plugin_basefile_path = __FILE__;
            $this->plugin_dir_path = plugin_dir_path(__FILE__) ;

            $this->plugin_markers_file_path = wp_upload_dir()['basedir'].'/'.$this->plugin_prefix.'markers/';
            $this->subs_geo_markers_filename = 'subscribers_locations.txt';

            $this->plugin_basefile = plugin_basename( $this->plugin_basefile_path );
            $this->plugin_url = plugin_dir_url( $this->plugin_basefile );
            $this->plugin_dir_path = trailingslashit( dirname( $this->plugin_basefile_path ) );
            $this->includes_path = $this->plugin_dir_path.'includes/';
            $this->plugin_textdomain = 'wpcoop' ;

            // templates
            $this->template_path_plugin = $this->plugin_dir_path . 'templates/';
            $this->template_path = $this->plugin_dir_path . 'templates/';
            $this->plugin_pdf_path = WP_PLUGIN_DIR. '/wpcoop/';
            $this->plugin_signatures_path = $this->plugin_dir_path. 'signatures/';

            //capability to read plugin data
            $this->plugin_read_capability=$this->plugin_prefix.'read_capability';
            //capability to edit plugin data
            $this->plugin_edit_capability=$this->plugin_prefix.'edit_capability';
            //access to plugin parameters not in options page
            $this->plugin_parameters_capability=$this->plugin_prefix.'parameters_capability';
            //access to options page
            $this->plugin_settings_capability='manage_options';

            $this->plugin_capability='manage_wpcoop';

            $this->plugin_lists_capability='lists_wpcoop';

            $this->plugin_usersubs_meta=$this->plugin_prefix."usersubs_meta";

            $this->subscribers_table=$this->plugin_prefix.'subscribers';

            $this->plugin_version= get_option($this->plugin_prefix.'plugin_version', '0.0');

            //wrapper for wlog not in muplugins
            if ( !function_exists('wlog') ) {
              function wlog($log, $text){}
            }
            if ( !function_exists('wlogt') ) {
              function wlogt($log, $text, $array){}
            }

            if ( !function_exists('print_tab') ) {
              function print_tab($var){
                echo '<pre>';
                print_r($var);
                echo '</pre>';
              }
            }

            if (!function_exists('write_log')) {

             function write_log($log) {
                if (true === WP_DEBUG) {
                   if (is_array($log) || is_object($log)) {
                     error_log(print_r($log, true));
                   } else {
                     error_log($log);
                   }
                }
              }
            }

            // Add the endpoints for the frontend
            // templates, ajax responder, css, and js must be located in $this->template_path_plugin.'plants' or sun ...
            // files are named with mode _ endpoin like ajax_plants or display_sun ...
            $this->api_endpoints = array(
                'myaccount' => get_option( $this->plugin_prefix . 'myaccount', 'mon-compte' ),
                'image' => get_option( $this->plugin_prefix . 'image', 'image' ),
                'subscribe' => get_option( $this->plugin_prefix . 'subscribe', 'souscrire' ),
                'prospect' => get_option( $this->plugin_prefix . 'prospect', 'contact' ),
                'report' => get_option( $this->plugin_prefix . 'report', 'rapport' )
            );

            /* activation/deactivation stuff */
            register_activation_hook( __FILE__, array($this,'on_activate' ) );
            register_deactivation_hook( __FILE__, array($this,'on_deactivate' ) );


            add_action( 'init', array( $this, 'plugin_init' ) );
            add_action( 'wp_loaded', array( $this, 'plugin_update' ) );

            //add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
            add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

            // redirect for endpoint
            add_action( 'template_redirect', array( $this, 'template_redirect_frontend' ) );
            //add_action( 'template_redirect', array( $this, 'find_template' ) );

            //methode pour supprimer 404 sur des query endpoint aso
            //add_filter('pre_handle_404', function($preempt, $wp_query) {
            //global $wp;
            // print_tab($wp_query);
            //print_tab($wp->request);

        }

        public function on_activate(){
          update_option($this->plugin_prefix.'haveto_flush_rewrite_rules', true);
          if(!$this->subscribers_table_exists()) $this->create_subscribers_table();
        }

        public function subscribers_table_exists(){
            global $wpdb;
            if(1==$wpdb->get_var(
                "SELECT count(*) FROM information_schema.tables
                 WHERE table_name = '".$wpdb->prefix . $this->subscribers_table."'")
              )
              return true ; else return false;
        }

        public function create_subscribers_table(){
           //write_log('create_subscribers_table');
           //wlog(LOG,'create subscribers table');
           global $wpdb;

           //https://codex.wordpress.org/Creating_Tables_with_Plugins
           $table_name = $wpdb->prefix . $this->subscribers_table;
           $charset_collate = $wpdb->get_charset_collate();
           $sql="
CREATE TABLE $table_name (
  id int(11) NOT NULL AUTO_INCREMENT,
  firstname varchar(255) NOT NULL,
  lastname varchar(255) NOT NULL,
  email varchar(255) DEFAULT NULL,
  birthdate date NOT NULL,
  zip varchar(6) NOT NULL,
  datecreated datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
  datemodified datetime NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
  PRIMARY KEY  (id)
) $charset_collate;";
           require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	       $result=dbDelta( $sql );
           //wlogt(LOG, 'dbdelta', $result);
        }

        public function on_deactivate(){
          //https://developer.wordpress.org/reference/classes/wp_rewrite/add_endpoint/
          global $wp, $wp_rewrite;
          $this->api_endpoints=apply_filters($this->plugin_prefix .'endpoints', $this->api_endpoints);
          foreach( $this->api_endpoints as $ep ) {
              //all endpoints in EP_ROOT );
              $k=array_search($ep, $wp->public_query_vars);
              if ($k!==false) unset($wp->public_query_vars[$k]);
              foreach($wp_rewrite->endpoints as $k=>$rep){
                if ($rep[0] == EP_ROOT && $rep[1]==$ep && $rep[2]==$ep) unset($wp_rewrite->endpoints[$k]);
              }
          }
           flush_rewrite_rules();
        }

        public function plugin_init() {

            // Internationalize the plugin
            $this->localise();
            // Load includes
            $this->includes();
            // load resources (js, css, aso...)
            $this->load_resources();
            $this->functions= new WPCOOP_Functions;
            $this->members= new WPCOOP_Members;
            $this->shares= new WPCOOP_Shares;
            // settings must be in first position
            $this->admin= new WPCOOP_Admin;
            $this->admin_params= new WPCOOP_Admin_Parameters;
            $this->settings= new WPCOOP_Admin_Settings;
            //$this->stripe= new WordPress_Cooperative_Stripe;

            // add endpoints
            $this->add_endpoints();

        }
        /*
        public function plugin_settings() {
            // get version
            $new_version=$this->plugin_version();
            $old_version=get_option($this->plugin_prefix.'plugin_version');
            // update if new version
            // cancel plugin if previous
            // @TODO Error message
            if($this->is_later_version($new_version, $old_version)) {
              $this->update($new_version,$old_version);
              update_option($this->plugin_prefix.'plugin_version', $new_version );
            } elseif ($this->is_previous_version($new_version, $old_version)) {
              //@TODO manage old versions
              return;
            }

            $this->plugin_version=$new_version;
        }
*/
        public function localise() {
            // Load language files from the plugin folder
            load_plugin_textdomain( $this->plugin_textdomain, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
            //load_plugin_textdomain('wp-fullcalendar', false, dirname( plugin_basename( __FILE__ ) ).'/includes/langs');
            // Internationalize the plugin with possibility to change locale
            //$locale = apply_filters( 'plugin_locale', get_locale(), $this->plugin_textdomain);
            //load_textdomain($this->plugin_textdomain, $this->plugin_dir_path . 'languages/.$this->plugin_textdomain.'-'.$locale.'.mo');
        }

        public function load_resources() {

            // add or remove users capabilities
            // array of users to add or remove capabilities
            // almtest=2, serge =3
            $cap_users=array(3);
            // capabilities to add or remove
            $caps = array(
                'manage_wpcoop'
                );
            foreach ($cap_users as $userid) {
                $user = new WP_User( $userid );
                foreach ( $caps as $cap ) {
                    $user->add_cap( $cap );
                    //$user->remove_cap( $cap );
                }
            }

        }

        public function enqueue_scripts()  {
         wp_enqueue_script('wpcoop', $this->plugin_url.'js/wpcoop.js', array('jquery'));
        }

        public function includes() {
          include_once( $this->includes_path.'class-wpcoop-functions.php' );
          include_once( $this->includes_path.'class-wpcoop-members.php' );
          include_once( $this->includes_path.'class-wpcoop-member.php' );
          include_once( $this->includes_path.'class-wpcoop-shares.php' );
          include_once( $this->includes_path.'class-wpcoop-share.php' );
          //include_once( $this->includes_path.'class-wpcoop-stripe.php' );
          //include_once( $this->includes_path.'proxy_forms.php' );

          include_once( $this->includes_path.'class-wpcoop-admin.php' );
          include_once( $this->includes_path.'class-wpcoop-admin-settings.php' );
          include_once( $this->includes_path.'class-wpcoop-admin_params.php' );
          //include_once( $this->includes_path.'class-wpcoop-test.php' );

          //include_once( 'includes/class-wpcoop-stripe.php' );
          include_once( $this->includes_path.'proxy_forms.php' );
        }


        /**
         * Add endpoints for query vars.
         * the endpoint is used in the front-end to
         * generate the template and link.
         */
        public function add_endpoints() {
            $this->api_endpoints=apply_filters($this->plugin_prefix .'endpoints', $this->api_endpoints);
            foreach( $this->api_endpoints as $ep ) {
                add_rewrite_endpoint( $ep, EP_ROOT );
            }
            // Flush the rules when have to
            // This is important to make the endpoints work.
            if( get_option( $this->plugin_prefix . 'haveto_flush_rewrite_rules' ) == true ) {
              update_option( $this->plugin_prefix . 'haveto_flush_rewrite_rules', false );
              flush_rewrite_rules();
            }
        }

        public function template_redirect_frontend() {
            global $wp;
            //print_r($wp->query_vars);
            // Check the page url and display the template if query-vars ok
            // end point ep and and point key
            foreach( $this->api_endpoints as $epk=>$ep ) {
                //print_tab($wp->query_vars);
                if( isset($wp->query_vars[$ep]) && (!empty( $wp->query_vars[$ep] || $wp->query_vars[$ep]==0 )) ) {
                    $this->generate_template( $epk, $wp->query_vars[$ep]);
                }
            }
        }

        /**
         * Generate the template
         */
        public function generate_template( $epk, $val ) {
            // epk end point key
            // $val value to be known by template
            $thispageurl=site_url('/'.$this->api_endpoints[$epk].'/'.$val,'https');
            if( $this->check_permissions() ) {
                // Load the template
                if (file_exists($this->template_path_plugin.$epk.'.php')){
                 include($this->template_path_plugin.$epk.'.php');
                 exit;
                }
            }
        }

       /**
         * Check the permissions
         */
        private function check_permissions () {
                // temporary check
                // restrict to logged in users
                //if( !is_user_logged_in()) {
                //    return false;
                //}

                // user is logged in
            // check must be done in templates
            return true;
        }



        public function plugin_update() {
            // get version
            if (!is_admin()) return;
            $new_version=$this->plugin_version();
            $old_version=get_option($this->plugin_prefix.'plugin_version');
            // update if new version
            // cancel plugin if previous
            // @TODO Error message
            if($this->is_later_version($new_version, $old_version)) {
              $this->update($new_version,$old_version);

            } elseif ($this->is_previous_version($new_version, $old_version)) {
              esc_html(printf(__('Impossible to downgrade WPCOOP version %1$s to %2$s, contact your system manager',$this->plugin_textdomain), $old_version, $new_version));
              exit;
            }
        }

        public function plugin_version() {
            if( !function_exists('get_plugin_data') ){
                require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
            }
            $plugin_data = get_plugin_data( __FILE__ );
            return $plugin_data['Version'];
        }

        private function is_later_version($new, $old) {
          return $new>$old;
        }

        private function is_previous_version($new, $old) {
          return $new<$old;
        }

        private function update($new, $old) {
         //echo '$new, $old: '."$new, $old<br>";
         switch ($old) {
             case '0.21':
                switch ($new) {
                    case '0.22':
                      break;
                    case '0.23':
                        $user_data_fields=$this->admin->get_user_data_fields();
                        //print_tab($user_data_fields);

                        $added_profile_fields=$this->admin->get_added_profile_fields();
                        //print_tab($added_profile_fields);

                        $added_profile_category_fields=$this->admin->get_added_profile_category_fields();
                        //print_tab($added_profile_category_fields);

                        $users_added_fields= $this->admin->get_users_added_fields();
                        //print_tab($users_added_fields);

                        $users=$this->admin->get_users(false, 'subs');
                        //print_tab($users);
                        global $wpdb;
                        $qry="DELETE FROM ".$wpdb->usermeta.' WHERE meta_key LIKE "'.$this->plugin_prefix.$this->usersdata_prefix.'%"';
                        //echo $qry.'<br>';
                        $wpdb->query($qry);

                        foreach($users as $user) {
                            /*
                          // fill   user_data
                          $userObj = get_userdata( $user->ID);


                          $user_data[$user->ID]['roles']=$userObj->roles;
                          $user_data[$user->ID]['meta']=get_user_meta($user->ID);
                          $user_data[$user->ID]['meta']['ID'][0]=$userObj->data->ID;
                          $user_data[$user->ID]['meta']['user_login'][0]=$userObj->data->user_login;
                          $user_data[$user->ID]['meta']['user_nicename'][0]=$userObj->data->user_nicename;
                          $user_data[$user->ID]['meta']['user_email'][0]=$userObj->data->user_email;
                          $user_data[$user->ID]['meta']['user_url'][0]=$userObj->data->user_url;
                          $user_data[$user->ID]['meta']['display_name'][0]=$userObj->data->display_name;
//                          echo '-------------';
//                          print_tab($userObj);
//                          print_tab($user_data[$user->ID]);
//                          echo '-------------';
//                          exit;
                         */
                          //echo 'user ID: '.$user->ID.'<br>';

                          //$user_data[$user->ID]['meta']=get_user_meta($user->ID);
                          $usermeta=get_user_meta($user->ID);
                          //print_tab($usermeta);
                          //echo '------------------ company name: '.$user_data[$user->ID]['meta'][$this->plugin_prefix.'company'][0].'<br>';

                          if (!empty($usermeta[$this->plugin_prefix.'company'][0])) {
                            // subscriber is an institution, set member_type  to 'i'
                            // fill legal representative data
                            update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'membertype'.'_type','i');
                            foreach($usermeta as $k=>$val) {
                              if (substr($k,0, strlen($this->plugin_prefix.$this->usersdata_prefix))==$this->plugin_prefix.$this->usersdata_prefix) continue;
                              if (substr($k,0, strlen($this->plugin_prefix))!=$this->plugin_prefix) {
                                // where meta index doesn't begin with plugin_prefix
                                switch($k) {
                                    case 'user_url':
                                      update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'representative'.'_user_url', $val[0]);
                                      break;
                                    case 'user_email':
                                      update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'representative'.'_user_email', $val[0]);
                                      break;
                                    case 'first_name':
                                      update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'representative'.'_first_name', $val[0]);
                                      break;
                                    case 'last_name':
                                      update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'representative'.'_last_name', $val[0]);
                                      break;
                                    default:
                                        // no significant meta data for this, continue
                                        //continue;
                                }
                              } else {
                              // where meta index begins with plugin_prefix
                              switch(substr($k, strlen($this->plugin_prefix))) {
                                  case 'forme':
                                   //update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'memberinstitution'.'_legalform', $val[0]);
                                   $meta=$this->plugin_prefix.$this->usersdata_prefix.'memberinstitution'.'_legalform';
                                   break;
                                  case 'company':
                                   //update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'memberinstitution'.'_name', $val[0]);
                                   $meta=$this->plugin_prefix.$this->usersdata_prefix.'memberinstitution'.'_name';
                                   break;
                                  case 'birthcountry':
                                   //update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'memberinstitution'.'_name', $val[0]);
                                   $meta=$this->plugin_prefix.$this->usersdata_prefix.'memberinstitution'.'_birthplace';
                                   break;
                                  case 'usersubs_meta':
                                   // no change for this
                                   $meta='';
                                   break;
                                  default:
                                   //field old name == new name
                                   //update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'representative'.'_'.substr($k, strlen($this->plugin_prefix)), $val[0]);
                                   $meta=$this->plugin_prefix.$this->usersdata_prefix.'representative'.'_'.substr($k, strlen($this->plugin_prefix));
                              }
                              if ($meta!='') {
                                  update_user_meta($user->ID, $meta, $val[0]);
                                  //echo $meta.' set to : '.$val[0].'<br>';
                              }
                             }
                            } // foreach
                          } else {
                            // subscriber is a person, set member_type  to 'c' (previous version doesn't recognise minors
                            update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'membertype'.'_type','c');
                            // subscriber is not a company, fill  data
                            foreach($usermeta as $k=>$val) {
                               switch($k) {
                                    case 'user_url':
                                      update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'memberperson'.'_user_url', $val[0]);
                                      break;
                                    case 'user_email':
                                      update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'memberperson'.'_user_email', $val[0]);
                                      break;
                                    case 'first_name':
                                      update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'memberperson'.'_first_name', $val[0]);
                                      break;
                                    case 'last_name':
                                      update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'memberperson'.'_last_name', $val[0]);
                                      break;
                                    default:
                                        // no significant meta data for this, continue
                                        //continue;
                                }

                              // ignore non plugin meta and plugin new fields
                              if (substr($k,0, strlen($this->plugin_prefix))!=$this->plugin_prefix) continue;
                              if (substr($k,0, strlen($this->plugin_prefix.$this->usersdata_prefix))==$this->plugin_prefix.$this->usersdata_prefix) continue;
                              // set fields with same names
                              switch (substr($k, strlen($this->plugin_prefix))){
                               case 'birthcountry':
                                $meta=$this->plugin_prefix.$this->usersdata_prefix.'memberperson'.'_birthplace';
                                break;
                               case 'usersubs_meta':
                                // no change for this
                                $meta='';
                                break;
                               default:
                                $meta=$this->plugin_prefix.$this->usersdata_prefix.'memberperson'.'_'.substr($k, strlen($this->plugin_prefix));
                              }
                              //update_user_meta($user->ID, $this->plugin_prefix.$this->usersdata_prefix.'memberperson'.'_'.substr($k, strlen($this->plugin_prefix)), $val[0]);
                              if ($meta!='') {
                                  update_user_meta($user->ID, $meta, $val[0]);
                                  //echo $meta.' set to : '.$val[0].'<br>';
                              }

                            }
                          }
                        }

                    break;
                } // $new

              break;
            } // old

            // update plugin version
            update_option($this->plugin_prefix.'plugin_version', $new );
            $this->plugin_version=$new;

        } // end of update method

   } //end of WordPress_Cooperative class
}// if exists WordPress_Cooperative class

// create instance
$wpcoop = new WordPress_Cooperative;
